@@ -15,6 +15,7 @@
  */
 
 
+#include "mongoc-aggregate-private.h"
 #include "mongoc-client-private.h"
 #include "mongoc-collection.h"
 #include "mongoc-collection-private.h"
@@ -64,8 +65,8 @@ _mongoc_database_new (mongoc_client_t *client,
 
    ENTRY;
 
-   BSON_ASSERT (client);
-   BSON_ASSERT (name);
+   BSON_ASSERT_PARAM (client);
+   BSON_ASSERT_PARAM (name);
 
    db = (mongoc_database_t *) bson_malloc0 (sizeof *db);
    db->client = client;
@@ -76,7 +77,7 @@ _mongoc_database_new (mongoc_client_t *client,
    db->read_prefs = read_prefs ? mongoc_read_prefs_copy (read_prefs)
                                : mongoc_read_prefs_new (MONGOC_READ_PRIMARY);
 
-   bson_strncpy (db->name, name, sizeof db->name);
+   db->name = bson_strdup (name);
 
    RETURN (db);
 }
@@ -122,11 +123,31 @@ mongoc_database_destroy (mongoc_database_t *database)
       database->write_concern = NULL;
    }
 
+   bson_free (database->name);
    bson_free (database);
 
    EXIT;
 }
 
+
+mongoc_cursor_t *
+mongoc_database_aggregate (mongoc_database_t *db,                 /* IN */
+                           const bson_t *pipeline,                /* IN */
+                           const bson_t *opts,                    /* IN */
+                           const mongoc_read_prefs_t *read_prefs) /* IN */
+{
+   return _mongoc_aggregate (db->client,
+                             db->name,
+                             MONGOC_QUERY_NONE,
+                             pipeline,
+                             opts,
+                             read_prefs,
+                             db->read_prefs,
+                             db->read_concern,
+                             db->write_concern);
+}
+
+
 /*
  *--------------------------------------------------------------------------
  *
@@ -149,7 +170,7 @@ mongoc_database_copy (mongoc_database_t *database)
 {
    ENTRY;
 
-   BSON_ASSERT (database);
+   BSON_ASSERT_PARAM (database);
 
    RETURN (_mongoc_database_new (database->client,
                                  database->name,
@@ -168,12 +189,19 @@ mongoc_database_command (mongoc_database_t *database,
                          const bson_t *fields,
                          const mongoc_read_prefs_t *read_prefs)
 {
-   char ns[MONGOC_NAMESPACE_MAX];
+   char *ns;
+   mongoc_cursor_t *cursor;
 
-   BSON_ASSERT (database);
-   BSON_ASSERT (command);
+   BSON_UNUSED (flags);
+   BSON_UNUSED (skip);
+   BSON_UNUSED (limit);
+   BSON_UNUSED (batch_size);
+   BSON_UNUSED (fields);
 
-   bson_snprintf (ns, sizeof ns, "%s.$cmd", database->name);
+   BSON_ASSERT_PARAM (database);
+   BSON_ASSERT_PARAM (command);
+
+   ns = bson_strdup_printf ("%s.$cmd", database->name);
 
    /* Server Selection Spec: "The generic command method has a default read
     * preference of mode 'primary'. The generic command method MUST ignore any
@@ -183,8 +211,10 @@ mongoc_database_command (mongoc_database_t *database,
     */
 
    /* flags, skip, limit, batch_size, fields are unused */
-   return _mongoc_cursor_cmd_deprecated_new (
+   cursor = _mongoc_cursor_cmd_deprecated_new (
       database->client, ns, command, read_prefs);
+   bson_free (ns);
+   return cursor;
 }
 
 
@@ -195,8 +225,8 @@ mongoc_database_command_simple (mongoc_database_t *database,
                                 bson_t *reply,
                                 bson_error_t *error)
 {
-   BSON_ASSERT (database);
-   BSON_ASSERT (command);
+   BSON_ASSERT_PARAM (database);
+   BSON_ASSERT_PARAM (command);
 
    /* Server Selection Spec: "The generic command method has a default read
     * preference of mode 'primary'. The generic command method MUST ignore any
@@ -208,7 +238,7 @@ mongoc_database_command_simple (mongoc_database_t *database,
    return _mongoc_client_command_with_opts (database->client,
                                             database->name,
                                             command,
-                                            MONGOC_CMD_READ,
+                                            MONGOC_CMD_RAW,
                                             NULL /* opts */,
                                             MONGOC_QUERY_NONE,
                                             read_prefs,
@@ -346,7 +376,7 @@ mongoc_database_drop_with_opts (mongoc_database_t *database,
    bool ret;
    bson_t cmd;
 
-   BSON_ASSERT (database);
+   BSON_ASSERT_PARAM (database);
 
    bson_init (&cmd);
    bson_append_int32 (&cmd, "dropDatabase", 12, 1);
@@ -379,8 +409,8 @@ mongoc_database_remove_user (mongoc_database_t *database,
 
    ENTRY;
 
-   BSON_ASSERT (database);
-   BSON_ASSERT (username);
+   BSON_ASSERT_PARAM (database);
+   BSON_ASSERT_PARAM (username);
 
    bson_init (&cmd);
    BSON_APPEND_UTF8 (&cmd, "dropUser", username);
@@ -400,7 +430,7 @@ mongoc_database_remove_all_users (mongoc_database_t *database,
 
    ENTRY;
 
-   BSON_ASSERT (database);
+   BSON_ASSERT_PARAM (database);
 
    bson_init (&cmd);
    BSON_APPEND_INT32 (&cmd, "dropAllUsersFromDatabase", 1);
@@ -439,8 +469,8 @@ mongoc_database_add_user (mongoc_database_t *database,
 
    ENTRY;
 
-   BSON_ASSERT (database);
-   BSON_ASSERT (username);
+   BSON_ASSERT_PARAM (database);
+   BSON_ASSERT_PARAM (username);
 
    bson_init (&cmd);
    BSON_APPEND_UTF8 (&cmd, "createUser", username);
@@ -483,7 +513,7 @@ mongoc_database_add_user (mongoc_database_t *database,
 const mongoc_read_prefs_t *
 mongoc_database_get_read_prefs (const mongoc_database_t *database) /* IN */
 {
-   BSON_ASSERT (database);
+   BSON_ASSERT_PARAM (database);
    return database->read_prefs;
 }
 
@@ -508,7 +538,7 @@ void
 mongoc_database_set_read_prefs (mongoc_database_t *database,
                                 const mongoc_read_prefs_t *read_prefs)
 {
-   BSON_ASSERT (database);
+   BSON_ASSERT_PARAM (database);
 
    if (database->read_prefs) {
       mongoc_read_prefs_destroy (database->read_prefs);
@@ -540,7 +570,7 @@ mongoc_database_set_read_prefs (mongoc_database_t *database,
 const mongoc_read_concern_t *
 mongoc_database_get_read_concern (const mongoc_database_t *database)
 {
-   BSON_ASSERT (database);
+   BSON_ASSERT_PARAM (database);
 
    return database->read_concern;
 }
@@ -566,7 +596,7 @@ void
 mongoc_database_set_read_concern (mongoc_database_t *database,
                                   const mongoc_read_concern_t *read_concern)
 {
-   BSON_ASSERT (database);
+   BSON_ASSERT_PARAM (database);
 
    if (database->read_concern) {
       mongoc_read_concern_destroy (database->read_concern);
@@ -598,7 +628,7 @@ mongoc_database_set_read_concern (mongoc_database_t *database,
 const mongoc_write_concern_t *
 mongoc_database_get_write_concern (const mongoc_database_t *database)
 {
-   BSON_ASSERT (database);
+   BSON_ASSERT_PARAM (database);
 
    return database->write_concern;
 }
@@ -624,7 +654,7 @@ void
 mongoc_database_set_write_concern (mongoc_database_t *database,
                                    const mongoc_write_concern_t *write_concern)
 {
-   BSON_ASSERT (database);
+   BSON_ASSERT_PARAM (database);
 
    if (database->write_concern) {
       mongoc_write_concern_destroy (database->write_concern);
@@ -669,8 +699,8 @@ mongoc_database_has_collection (mongoc_database_t *database,
 
    ENTRY;
 
-   BSON_ASSERT (database);
-   BSON_ASSERT (name);
+   BSON_ASSERT_PARAM (database);
+   BSON_ASSERT_PARAM (name);
 
    if (error) {
       memset (error, 0, sizeof *error);
@@ -711,7 +741,7 @@ mongoc_database_find_collections (mongoc_database_t *database,
    bson_t opts = BSON_INITIALIZER;
    mongoc_cursor_t *cursor;
 
-   BSON_ASSERT (database);
+   BSON_ASSERT_PARAM (database);
 
    if (filter) {
       if (!BSON_APPEND_DOCUMENT (&opts, "filter", filter)) {
@@ -745,7 +775,7 @@ mongoc_database_find_collections_with_opts (mongoc_database_t *database,
    mongoc_cursor_t *cursor;
    bson_t cmd = BSON_INITIALIZER;
 
-   BSON_ASSERT (database);
+   BSON_ASSERT_PARAM (database);
 
    BSON_APPEND_INT32 (&cmd, "listCollections", 1);
 
@@ -785,7 +815,7 @@ mongoc_database_get_collection_names_with_opts (mongoc_database_t *database,
    const bson_t *doc;
    char **ret;
 
-   BSON_ASSERT (database);
+   BSON_ASSERT_PARAM (database);
 
    if (opts) {
       bson_copy_to (opts, &opts_copy);
@@ -829,20 +859,19 @@ mongoc_database_get_collection_names_with_opts (mongoc_database_t *database,
    return ret;
 }
 
-
-mongoc_collection_t *
-mongoc_database_create_collection (mongoc_database_t *database,
-                                   const char *name,
-                                   const bson_t *opts,
-                                   bson_error_t *error)
+static mongoc_collection_t *
+create_collection (mongoc_database_t *database,
+                   const char *name,
+                   const bson_t *opts,
+                   bson_error_t *error)
 {
    mongoc_collection_t *collection = NULL;
    bson_iter_t iter;
    bson_t cmd;
    bool capped = false;
 
-   BSON_ASSERT (database);
-   BSON_ASSERT (name);
+   BSON_ASSERT_PARAM (database);
+   BSON_ASSERT_PARAM (name);
 
    if (strchr (name, '$')) {
       bson_set_error (error,
@@ -972,13 +1001,307 @@ mongoc_database_create_collection (mongoc_database_t *database,
    return collection;
 }
 
+char *
+_mongoc_get_encryptedField_state_collection (
+   const bson_t *encryptedFields,
+   const char *data_collection,
+   const char *state_collection_suffix,
+   bson_error_t *error)
+{
+   bson_iter_t iter;
+   const char *fieldName = NULL;
+
+   if (0 == strcmp (state_collection_suffix, "esc")) {
+      fieldName = "escCollection";
+   } else if (0 == strcmp (state_collection_suffix, "ecc")) {
+      fieldName = "eccCollection";
+   } else if (0 == strcmp (state_collection_suffix, "ecoc")) {
+      fieldName = "ecocCollection";
+   } else {
+      bson_set_error (error,
+                      MONGOC_ERROR_COMMAND,
+                      MONGOC_ERROR_COMMAND_INVALID_ARG,
+                      "expected state_collection_suffix to be 'esc', 'ecc', or "
+                      "'ecoc', got: %s",
+                      state_collection_suffix);
+      return NULL;
+   }
+
+   if (bson_iter_init_find (&iter, encryptedFields, fieldName)) {
+      if (!BSON_ITER_HOLDS_UTF8 (&iter)) {
+         bson_set_error (error,
+                         MONGOC_ERROR_COMMAND,
+                         MONGOC_ERROR_COMMAND_INVALID_ARG,
+                         "expected encryptedFields.%s to be UTF-8",
+                         fieldName);
+         return NULL;
+      }
+      return bson_strdup (bson_iter_utf8 (&iter, NULL));
+   }
+   return bson_strdup_printf (
+      "enxcol_.%s.%s", data_collection, state_collection_suffix);
+}
+
+static bool
+create_encField_state_collection (mongoc_database_t *database,
+                                  const bson_t *encryptedFields,
+                                  const char *data_collection,
+                                  const char *state_collection_suffix,
+                                  bson_error_t *error)
+{
+   char *state_collection = NULL;
+   mongoc_collection_t *collection = NULL;
+   bool ok = false;
+   bson_t opts = BSON_INITIALIZER;
+
+   state_collection = _mongoc_get_encryptedField_state_collection (
+      encryptedFields, data_collection, state_collection_suffix, error);
+   if (!state_collection) {
+      goto fail;
+   }
+
+   BCON_APPEND (&opts,
+                "clusteredIndex",
+                "{",
+                "key",
+                "{",
+                "_id",
+                BCON_INT32 (1),
+                "}",
+                "unique",
+                BCON_BOOL (true),
+                "}");
+
+   collection = create_collection (database, state_collection, &opts, error);
+   if (collection == NULL) {
+      goto fail;
+   }
+
+   ok = true;
+fail:
+   bson_free (state_collection);
+   mongoc_collection_destroy (collection);
+   bson_destroy (&opts);
+   return ok;
+}
+
+static mongoc_collection_t *
+create_collection_with_encryptedFields (mongoc_database_t *database,
+                                        const char *name,
+                                        const bson_t *opts,
+                                        const bson_t *encryptedFields,
+                                        bson_error_t *error)
+{
+   mongoc_collection_t *dataCollection = NULL;
+   bool ok = false;
+   bson_t *cc_opts = NULL;
+   bool state_collections_ok =
+      create_encField_state_collection (
+         database, encryptedFields, name, "esc", error) &&
+      create_encField_state_collection (
+         database, encryptedFields, name, "ecc", error) &&
+      create_encField_state_collection (
+         database, encryptedFields, name, "ecoc", error);
+   if (!state_collections_ok) {
+      // Failed to create one or more state collections
+      goto fail;
+   }
+
+   /* Create data collection. */
+   cc_opts = bson_copy (opts);
+   if (!BSON_APPEND_DOCUMENT (cc_opts, "encryptedFields", encryptedFields)) {
+      bson_set_error (error,
+                      MONGOC_ERROR_COMMAND,
+                      MONGOC_ERROR_COMMAND_INVALID_ARG,
+                      "unable to append encryptedFields");
+      goto fail;
+   }
+   dataCollection = create_collection (database, name, cc_opts, error);
+   if (!dataCollection) {
+      goto fail;
+   }
+
+   /* Create index on __safeContent__. */
+   {
+      bson_t *keys = BCON_NEW ("__safeContent__", BCON_INT32 (1));
+      char *index_name;
+      bson_t *create_indexes;
+
+      index_name = mongoc_collection_keys_to_index_string (keys);
+      create_indexes = BCON_NEW ("createIndexes",
+                                 BCON_UTF8 (name),
+                                 "indexes",
+                                 "[",
+                                 "{",
+                                 "key",
+                                 BCON_DOCUMENT (keys),
+                                 "name",
+                                 BCON_UTF8 (index_name),
+                                 "}",
+                                 "]");
+
+      ok = mongoc_database_write_command_with_opts (
+         database, create_indexes, NULL /* opts */, NULL /* reply */, error);
+      bson_destroy (create_indexes);
+      bson_free (index_name);
+      bson_destroy (keys);
+      if (!ok) {
+         goto fail;
+      }
+   }
+
+   ok = true;
+fail:
+   bson_destroy (cc_opts);
+   if (ok) {
+      return dataCollection;
+   } else {
+      mongoc_collection_destroy (dataCollection);
+      return NULL;
+   }
+}
+
+bool
+_mongoc_get_encryptedFields_from_map (mongoc_client_t *client,
+                                      const char *dbName,
+                                      const char *collName,
+                                      bson_t *encryptedFields,
+                                      bson_error_t *error)
+{
+   const bson_t *efMap = client->topology->encrypted_fields_map;
+
+   bson_init (encryptedFields);
+
+   if (bson_empty0 (efMap)) {
+      /* Unset or empty efMap will have no encrypted fields */
+      return true;
+   }
+
+   char *ns = bson_strdup_printf ("%s.%s", dbName, collName);
+   bson_iter_t iter;
+   if (!bson_iter_init_find (&iter, efMap, ns)) {
+      /* No efMap entry for this database+collection. */
+      bson_free (ns);
+      return true;
+   }
+   bson_free (ns);
+
+   if (!_mongoc_iter_document_as_bson (&iter, encryptedFields, error)) {
+      /* The efMap entry should always be a document. */
+      return false;
+   }
+
+   return true;
+}
+
+bool
+_mongoc_get_encryptedFields_from_server (mongoc_client_t *client,
+                                         const char *dbName,
+                                         const char *collName,
+                                         bson_t *encryptedFields,
+                                         bson_error_t *error)
+{
+   mongoc_database_t *db = mongoc_client_get_database (client, dbName);
+   bson_t *opts = BCON_NEW ("filter", "{", "name", BCON_UTF8 (collName), "}");
+   mongoc_cursor_t *cursor;
+   bool ret = false;
+   const bson_t *collInfo;
+
+   bson_init (encryptedFields);
+
+   cursor = mongoc_database_find_collections_with_opts (db, opts);
+   if (mongoc_cursor_error (cursor, error)) {
+      goto fail;
+   }
+
+   if (mongoc_cursor_next (cursor, &collInfo)) {
+      /* Check if the collInfo has options.encryptedFields. */
+      bson_iter_t iter;
+      if (!bson_iter_init (&iter, collInfo)) {
+         bson_set_error (error,
+                         MONGOC_ERROR_COMMAND,
+                         MONGOC_ERROR_COMMAND_INVALID_ARG,
+                         "unable to iterate listCollections result");
+         goto fail;
+      }
+
+      if (bson_iter_find_descendant (&iter, "options.encryptedFields", &iter)) {
+         bson_t tmp;
+         if (!_mongoc_iter_document_as_bson (&iter, &tmp, error)) {
+            goto fail;
+         }
+         bson_copy_to (&tmp, encryptedFields);
+      }
+   }
+
+   if (mongoc_cursor_error (cursor, error)) {
+      goto fail;
+   }
+
+   ret = true;
+fail:
+   mongoc_cursor_destroy (cursor);
+   bson_destroy (opts);
+   mongoc_database_destroy (db);
+   return ret;
+}
+
+mongoc_collection_t *
+mongoc_database_create_collection (mongoc_database_t *database,
+                                   const char *name,
+                                   const bson_t *opts,
+                                   bson_error_t *error)
+{
+   bson_iter_t iter;
+   bson_t encryptedFields = BSON_INITIALIZER;
+
+   if (opts && bson_iter_init_find (&iter, opts, "encryptedFields")) {
+      if (!_mongoc_iter_document_as_bson (&iter, &encryptedFields, error)) {
+         return NULL;
+      }
+   }
+
+   if (bson_empty (&encryptedFields)) {
+      if (!_mongoc_get_encryptedFields_from_map (
+             database->client,
+             mongoc_database_get_name (database),
+             name,
+             &encryptedFields,
+             error)) {
+         return NULL;
+      }
+   }
+
+   if (!bson_empty (&encryptedFields)) {
+      bson_t opts_without_encryptedFields = BSON_INITIALIZER;
+
+      if (opts) {
+         bson_copy_to_excluding_noinit (
+            opts, &opts_without_encryptedFields, "encryptedFields", NULL);
+      }
+
+      mongoc_collection_t *ret =
+         create_collection_with_encryptedFields (database,
+                                                 name,
+                                                 &opts_without_encryptedFields,
+                                                 &encryptedFields,
+                                                 error);
+
+      bson_destroy (&opts_without_encryptedFields);
+      bson_destroy (&encryptedFields);
+      return ret;
+   }
+
+   return create_collection (database, name, opts, error);
+}
+
 
 mongoc_collection_t *
 mongoc_database_get_collection (mongoc_database_t *database,
                                 const char *collection)
 {
-   BSON_ASSERT (database);
-   BSON_ASSERT (collection);
+   BSON_ASSERT_PARAM (database);
+   BSON_ASSERT_PARAM (collection);
 
    return _mongoc_collection_new (database->client,
                                   database->name,
@@ -992,7 +1315,7 @@ mongoc_database_get_collection (mongoc_database_t *database,
 const char *
 mongoc_database_get_name (mongoc_database_t *database)
 {
-   BSON_ASSERT (database);
+   BSON_ASSERT_PARAM (database);
 
    return database->name;
 }
