@@ -14,21 +14,22 @@
  * limitations under the License.
  */
 
+#include "mongoc-prelude.h"
+
 #ifndef MONGOC_ERRORS_H
 #define MONGOC_ERRORS_H
 
-#if !defined (MONGOC_INSIDE) && !defined (MONGOC_COMPILATION)
-# error "Only <mongoc.h> can be included directly."
-#endif
+#include <bson/bson.h>
 
-#include <bson.h>
+#include "mongoc-macros.h"
 
+#define MONGOC_ERROR_API_VERSION_LEGACY 1
+#define MONGOC_ERROR_API_VERSION_2 2
 
 BSON_BEGIN_DECLS
 
 
-typedef enum
-{
+typedef enum {
    MONGOC_ERROR_CLIENT = 1,
    MONGOC_ERROR_STREAM,
    MONGOC_ERROR_PROTOCOL,
@@ -45,11 +46,14 @@ typedef enum
    MONGOC_ERROR_SCRAM,
    MONGOC_ERROR_SERVER_SELECTION,
    MONGOC_ERROR_WRITE_CONCERN,
+   MONGOC_ERROR_SERVER, /* Error API Version 2 only */
+   MONGOC_ERROR_TRANSACTION,
+   MONGOC_ERROR_CLIENT_SIDE_ENCRYPTION, /* An error coming from libmongocrypt */
+   MONGOC_ERROR_POOL
 } mongoc_error_domain_t;
 
 
-typedef enum
-{
+typedef enum {
    MONGOC_ERROR_STREAM_INVALID_TYPE = 1,
    MONGOC_ERROR_STREAM_INVALID_STATE,
    MONGOC_ERROR_STREAM_NAME_RESOLUTION,
@@ -99,13 +103,41 @@ typedef enum
    MONGOC_ERROR_SERVER_SELECTION_INVALID_ID,
 
    MONGOC_ERROR_GRIDFS_CHUNK_MISSING,
+   MONGOC_ERROR_GRIDFS_PROTOCOL_ERROR,
 
    /* Dup with query failure. */
    MONGOC_ERROR_PROTOCOL_ERROR = 17,
 
    MONGOC_ERROR_WRITE_CONCERN_ERROR = 64,
+
+   MONGOC_ERROR_DUPLICATE_KEY = 11000,
+
+   MONGOC_ERROR_MAX_TIME_MS_EXPIRED = 50,
+
+   MONGOC_ERROR_CHANGE_STREAM_NO_RESUME_TOKEN,
+   MONGOC_ERROR_CLIENT_SESSION_FAILURE,
+   MONGOC_ERROR_TRANSACTION_INVALID_STATE,
+   MONGOC_ERROR_GRIDFS_CORRUPT,
+   MONGOC_ERROR_GRIDFS_BUCKET_FILE_NOT_FOUND,
+   MONGOC_ERROR_GRIDFS_BUCKET_STREAM,
+
+   /* An error related to initializing client side encryption. */
+   MONGOC_ERROR_CLIENT_INVALID_ENCRYPTION_STATE,
+
+   MONGOC_ERROR_CLIENT_INVALID_ENCRYPTION_ARG,
+
+
+   /* An error related to server version api */
+   MONGOC_ERROR_CLIENT_API_ALREADY_SET,
+   MONGOC_ERROR_CLIENT_API_FROM_POOL,
+   MONGOC_ERROR_POOL_API_ALREADY_SET,
+   MONGOC_ERROR_POOL_API_TOO_LATE,
+
+   MONGOC_ERROR_CLIENT_INVALID_LOAD_BALANCER,
 } mongoc_error_code_t;
 
+MONGOC_EXPORT (bool)
+mongoc_error_has_label (const bson_t *reply, const char *label);
 
 BSON_END_DECLS
 
