@@ -432,8 +432,8 @@ _mongoc_socket_setkeepalive_windows (SOCKET sd)
       TRACE ("%s", "Could not set keepalive values");
    } else {
       TRACE ("%s", "KeepAlive values updated");
-      TRACE ("KeepAliveTime: %d", keepalive.keepalivetime);
-      TRACE ("KeepAliveInterval: %d", keepalive.keepaliveinterval);
+      TRACE ("KeepAliveTime: %lu", keepalive.keepalivetime);
+      TRACE ("KeepAliveInterval: %lu", keepalive.keepaliveinterval);
    }
 }
 #else
@@ -557,7 +557,7 @@ static bool
 #ifdef _WIN32
 _mongoc_socket_setnodelay (SOCKET sd) /* IN */
 #else
-_mongoc_socket_setnodelay (int sd) /* IN */
+_mongoc_socket_setnodelay (int sd)   /* IN */
 #endif
 {
 #ifdef _WIN32
@@ -1023,7 +1023,7 @@ mongoc_socket_new (int domain,   /* IN */
    /* Set SO_NOSIGPIPE, to ignore SIGPIPE on writes for platforms where
       setting MSG_NOSIGNAL on writes is not supported (primarily OSX). */
 #ifdef SO_NOSIGPIPE
-   setsockopt(sd, SOL_SOCKET, SO_NOSIGPIPE, &on, sizeof(on));
+   setsockopt (sd, SOL_SOCKET, SO_NOSIGPIPE, &on, sizeof (on));
 #endif
 
    sock = (mongoc_socket_t *) bson_malloc0 (sizeof *sock);
@@ -1293,7 +1293,7 @@ _mongoc_socket_try_sendv (mongoc_socket_t *sock, /* IN */
 #ifdef _WIN32
    ret = WSASend (
       sock->sd, (LPWSABUF) iov, iovcnt, &dwNumberofBytesSent, 0, NULL, NULL);
-   TRACE ("WSASend sent: %ld (out of: %ld), ret: %d",
+   TRACE ("WSASend sent: %ld (out of: %zu), ret: %d",
           dwNumberofBytesSent,
           iov->iov_len,
           ret);
@@ -1389,7 +1389,7 @@ mongoc_socket_sendv (mongoc_socket_t *sock,  /* IN */
    for (;;) {
       sent = _mongoc_socket_try_sendv (sock, &iov[cur], iovcnt - cur);
       TRACE (
-         "Sent %ld (of %ld) out of iovcnt=%ld", sent, iov[cur].iov_len, iovcnt);
+         "Sent %zd (of %zu) out of iovcnt=%zu", sent, iov[cur].iov_len, iovcnt);
 
       /*
        * If we failed with anything other than EAGAIN or EWOULDBLOCK,
@@ -1414,7 +1414,7 @@ mongoc_socket_sendv (mongoc_socket_t *sock,  /* IN */
           * Subtract the sent amount from what we still need to send.
           */
          while ((cur < iovcnt) && (sent >= (ssize_t) iov[cur].iov_len)) {
-            TRACE ("still got bytes left: sent -= iov_len: %ld -= %ld",
+            TRACE ("still got bytes left: sent -= iov_len: %zd -= %zu",
                    sent,
                    iov[cur].iov_len);
             sent -= iov[cur++].iov_len;
@@ -1433,12 +1433,12 @@ mongoc_socket_sendv (mongoc_socket_t *sock,  /* IN */
           * Increment the current iovec buffer to its proper offset and adjust
           * the number of bytes to write.
           */
-         TRACE ("Seeked io_base+%ld", sent);
+         TRACE ("Seeked io_base+%zd", sent);
          TRACE (
-            "Subtracting iov_len -= sent; %ld -= %ld", iov[cur].iov_len, sent);
+            "Subtracting iov_len -= sent; %zu -= %zd", iov[cur].iov_len, sent);
          iov[cur].iov_base = ((char *) iov[cur].iov_base) + sent;
          iov[cur].iov_len -= sent;
-         TRACE ("iov_len remaining %ld", iov[cur].iov_len);
+         TRACE ("iov_len remaining %zu", iov[cur].iov_len);
 
          BSON_ASSERT (iovcnt - cur);
          BSON_ASSERT (iov[cur].iov_len);
